@@ -5,18 +5,9 @@ TODO: Need KgtkWriterOptions
 """
 
 from argparse import _MutuallyExclusiveGroup, Namespace, SUPPRESS
-from pathlib import Path
-import sys
 import typing
 
-from kgtk.kgtkformat import KgtkFormat
 from kgtk.cli_argparse import KGTKArgumentParser, KGTKFiles
-from kgtk.io.kgtkreader import KgtkReader, KgtkReaderOptions
-from kgtk.io.kgtkwriter import KgtkWriter
-from kgtk.reshape.kgtkexplode import KgtkExplode
-from kgtk.utils.argparsehelpers import optional_bool
-from kgtk.value.kgtkvalue import KgtkValueFields
-from kgtk.value.kgtkvalueoptions import KgtkValueOptions
 
 def parser():
     return {
@@ -35,6 +26,12 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
     Args:
         parser (argparse.ArgumentParser)
     """
+    from kgtk.kgtkformat import KgtkFormat
+    from kgtk.io.kgtkreader import KgtkReader, KgtkReaderOptions
+    from kgtk.io.kgtkwriter import KgtkWriter
+    from kgtk.utils.argparsehelpers import optional_bool
+    from kgtk.value.kgtkvalue import KgtkValueFields
+    from kgtk.value.kgtkvalueoptions import KgtkValueOptions
 
     _expert: bool = parsed_shared_args._expert
 
@@ -77,6 +74,9 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
                               help="Print the list of data types and exit. (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=False)
 
+    parser.add_argument(      "--output-format", dest="output_format", help="The file format (default=kgtk)", type=str,
+                              choices=KgtkWriter.OUTPUT_FORMAT_CHOICES)
+
     KgtkReader.add_debug_arguments(parser, expert=_expert)
     KgtkReaderOptions.add_arguments(parser, mode_options=True, expert=_expert)
     KgtkValueOptions.add_arguments(parser, expert=_expert)
@@ -90,6 +90,7 @@ def run(input_file: KGTKFiles,
         overwrite_columns: bool,
         expand_list: bool,
         show_data_types: bool,
+        output_format: typing.Optional[str],
         
         errors_to_stdout: bool = False,
         errors_to_stderr: bool = True,
@@ -100,7 +101,15 @@ def run(input_file: KGTKFiles,
         **kwargs # Whatever KgtkFileOptions and KgtkValueOptions want.
 )->int:
     # import modules locally
+    from pathlib import Path
+    import sys
+    
     from kgtk.exceptions import KGTKException
+    from kgtk.kgtkformat import KgtkFormat
+    from kgtk.io.kgtkreader import KgtkReader, KgtkReaderOptions
+    from kgtk.io.kgtkwriter import KgtkWriter
+    from kgtk.reshape.kgtkexplode import KgtkExplode
+    from kgtk.value.kgtkvalueoptions import KgtkValueOptions
 
     input_kgtk_file: Path = KGTKArgumentParser.get_input_file(input_file)
     output_kgtk_file: Path = KGTKArgumentParser.get_output_file(output_file)
@@ -125,6 +134,8 @@ def run(input_file: KGTKFiles,
         if field_names is not None:
             print("--fields %s" % " ".join(field_names), file=error_file, flush=True)
         print("--show-data-types %s" % str(show_data_types), file=error_file, flush=True)
+        if output_format is not None:
+            print("--output-format=%s" % output_format, file=error_file, flush=True)
         reader_options.show(out=error_file)
         value_options.show(out=error_file)
         print("=======", file=error_file, flush=True)
@@ -138,6 +149,7 @@ def run(input_file: KGTKFiles,
         ex: KgtkExplode = KgtkExplode(
             input_file_path=input_kgtk_file,
             output_file_path=output_kgtk_file,
+            output_format=output_format,
             column_name=column_name,
             type_names=type_names,
             field_names=field_names,
