@@ -151,6 +151,16 @@ def add_arguments(parser):
         help="set the path of the prefix kgtk file that provides customized uri prefix binding",
         dest="prefix_path",
     )
+    parser.add_argument(
+        "-i",
+        "--input-file",
+        action="store",
+        type=str,
+        required = False,
+        default="",
+        help="set the path of the input kgtk file if not from standard input",
+        dest="input_file",
+    )
 
 
 def run(
@@ -165,13 +175,15 @@ def run(
     use_id:bool,
     log_path:str,
     prop_declaration:bool,
-    prefix_path:str
+    prefix_path:str,
+    input_file: str,
 ):
     # import modules locally
     import gzip
     # from kgtk.triple_generator import TripleGenerator
     from kgtk.generator import TripleGenerator
     import sys
+    from kgtk.exceptions import KGTKException
 
     generator = TripleGenerator(
         prop_file=prop_file,
@@ -190,41 +202,62 @@ def run(
 
     # loop first round
     if use_gz:
-        fp = gzip.open(sys.stdin.buffer, 'rt')
+        if input_file:
+            try:
+                fp = open(prefix_path,"rb")
+            except:
+                raise KGTKException("Fail to read from compressed file {}. Exiting.".format(input_file))
+        else:
+            fp = gzip.open(sys.stdin.buffer, 'rt')
     else:
-        fp = sys.stdin
+        if input_file:
+            try:
+                fp = open(prefix_path,"r")
+            except:
+                raise KGTKException("Fail to read from file {}. Exiting.".format(input_file))
+        else:
+            fp = sys.stdin
         # not line by line
     
     if prop_declaration:
-        file_lines = 0
-        begining_edge = None
-        start_generation = False
-        for line_num, edge in enumerate(fp):
-            if line_num == 0:
-                begining_edge = edge
+        if input_file:
+            for line_num, edge in enumerate(fp):
+                generator.read_prop_declaration(line_num+1,edge)
+            fp.seek(0)
+            for line_num, edge in enumerate(fp):
                 generator.entry_point(line_num+1,edge)
-                file_lines += 1
-            else:
-                if start_generation:
-                    # start triple generation because reached the starting position of the second `cat`
-                    line_number = line_num - file_lines
-                    # print("creating triples at line {} {} with total number of lines: {}".format(line_number+1, edge, file_lines))
-                    generator.entry_point(line_number+1,edge) # file generator
-                    # print("# {}".format(generator.read_num_of_lines))
+        else:
+            file_lines = 0
+            begining_edge = None
+            start_generation = False
+            for line_num, edge in enumerate(fp):
+                if line_num == 0:
+                    begining_edge = edge
+                    generator.entry_point(line_num+1,edge)
+                    file_lines += 1
                 else:
-                    if edge == begining_edge:
-                        start_generation = True
+                    if start_generation:
+                        # start triple generation because reached the starting position of the second `cat`
+                        line_number = line_num - file_lines
+                        # print("creating triples at line {} {} with total number of lines: {}".format(line_number+1, edge, file_lines))
+                        generator.entry_point(line_number+1,edge) # file generator
+                        # print("# {}".format(generator.read_num_of_lines))
                     else:
-                        file_lines += 1
-                        # print("creating property declarations at line {} {}".format(line_num, edge))
-                        generator.read_prop_declaration(line_num+1,edge)
+                        if edge == begining_edge:
+                            start_generation = True
+                        else:
+                            file_lines += 1
+                            # print("creating property declarations at line {} {}".format(line_num, edge))
+                            generator.read_prop_declaration(line_num+1,edge)
                     
         generator.finalize()
     else:
+        # not declaration
         for line_num, edge in enumerate(fp):
             if edge.startswith("#") or len(edge.strip("\n")) == 0:
                 continue
             else:
                 generator.entry_point(line_num+1,edge)
-    
         generator.finalize()
+    if input_file:
+        fp.close()
